package compbio.stat.collector;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import compbio.ws.client.Services;

public class StatDBTester {
	StatDB statdb;

	@BeforeClass
	public void init() {
		try {
			statdb = new StatDB(true);
		} catch (SQLException e) {
			e.printStackTrace();
			Assert.fail(e.getLocalizedMessage());
		}
	}

	@Test
	public void testReadYearDataAndShutdown() {
		Calendar cal = Calendar.getInstance();
		cal.roll(Calendar.YEAR, false);
		try {
			List<JobStat> jobs = statdb.readData(
					new Timestamp(cal.getTimeInMillis()), new Timestamp(
							new Date().getTime()), Services.MuscleWS, false);
			assertNotNull(jobs);
			assertEquals(jobs.size(), 1294);
			statdb.shutdownDBServer();
		} catch (SQLException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		}
		// Test shutdown
		try {
			statdb.readData(new Timestamp(cal.getTimeInMillis()),
					new Timestamp(new Date().getTime()), Services.MuscleWS,
					false);
			Assert.fail("Should not reach this point");
		} catch (SQLException e) {
			assertEquals(e.getMessage(), "No current connection.");
		}

	}
	@Test
	public void testReadOneMonthData() {
		Calendar cal = Calendar.getInstance();
		cal.roll(Calendar.MONTH, false);
		try {
			List<JobStat> jobs = statdb.readData(
					new Timestamp(cal.getTimeInMillis()), new Timestamp(
							new Date().getTime()), Services.TcoffeeWS, false);
			assertNotNull(jobs);
			assertEquals(jobs.size(), 67);

			jobs = statdb.readData(new Timestamp(cal.getTimeInMillis()),
					new Timestamp(new Date().getTime()), Services.ClustalWS,
					false);

			assertNotNull(jobs);
			assertEquals(jobs.size(), 159);

			jobs = statdb.readData(new Timestamp(cal.getTimeInMillis()),
					new Timestamp(new Date().getTime()), Services.MafftWS,
					false);
			assertNotNull(jobs);
			assertEquals(jobs.size(), 163);

			jobs = statdb.readData(new Timestamp(cal.getTimeInMillis()),
					new Timestamp(new Date().getTime()), Services.ProbconsWS,
					false);
			assertNotNull(jobs);
			assertEquals(jobs.size(), 13);

			jobs = statdb.readData(new Timestamp(cal.getTimeInMillis()),
					new Timestamp(new Date().getTime()), Services.MuscleWS,
					false);
			assertNotNull(jobs);
			assertEquals(jobs.size(), 84);

		} catch (SQLException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		}
	}
}
